@@ -211,15 +211,36 @@ namespace ts.projectSystem {
             }, "/jsconfig.json");
         });
 
+        it("sends telemetry for file sizes", () => {
+            const jsFile = makeFile("/a.js", "1");
+            const tsFile = makeFile("/b.ts", "12");
+            const tsconfig = makeFile("/jsconfig.json", {
+                compilerOptions: autoJsCompilerOptions
+            });
+            const et = new TestServerEventManager([tsconfig, jsFile, tsFile]);
+            et.service.openClientFile(jsFile.path);
+            et.assertProjectInfoTelemetryEvent({
+                fileStats: fileStats({ js: 1, jsSize: 1, ts: 1, tsSize: 2 }),
+                compilerOptions: autoJsCompilerOptions,
+                typeAcquisition: {
+                    enable: true,
+                    include: false,
+                    exclude: false,
+                },
+                configFileName: "jsconfig.json",
+            }, "/jsconfig.json");
+        });
+
         it("detects whether language service was disabled", () => {
             const file = makeFile("/a.js");
             const tsconfig = makeFile("/jsconfig.json", {});
             const et = new TestServerEventManager([tsconfig, file]);
-            et.host.getFileSize = () => server.maxProgramSizeForNonTsFiles + 1;
+            const fileSize = server.maxProgramSizeForNonTsFiles + 1;
+            et.host.getFileSize = () => fileSize;
             et.service.openClientFile(file.path);
             et.getEvent<server.ProjectLanguageServiceStateEvent>(server.ProjectLanguageServiceStateEvent);
             et.assertProjectInfoTelemetryEvent({
-                fileStats: fileStats({ js: 1 }),
+                fileStats: fileStats({ js: 1, jsSize: fileSize }),
                 compilerOptions: autoJsCompilerOptions,
                 configFileName: "jsconfig.json",
                 typeAcquisition: {
