@@ -14,7 +14,7 @@ let f6 = async function(): MyPromise<void> { }
 
 let f7 = async () => { };
 let f8 = async (): Promise<void> => { };
-let f9 = async (): MyPromise<void> => { }; 
+let f9 = async (): MyPromise<void> => { };
 let f10 = async () => p;
 let f11 = async () => mp;
 let f12 = async (): Promise<number> => mp;
@@ -37,4 +37,11 @@ class C {
 
 module M {
 	export async function f1() { }
+}
+
+async function f14() {
+    block: {
+        await 1;
+        break block;
+    }
 }
\ No newline at end of file
